package cvi10;

import io.reactivex.Observable;
import io.reactivex.subjects.PublishSubject;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.HeadlessException;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class Whiteboard {
	
	private static final int HEIGHT = 480;
	private static final int WIDTH = 640;
	
	protected final JFrame frame = new JFrame();
	protected BufferedImage canvas = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
	protected PublishSubject<DrawMessage> drawMessages = PublishSubject.create();
	
	public Whiteboard() throws HeadlessException {
		super();
		
		frame.setSize(WIDTH, HEIGHT);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		prepareDrawing();
		drawMessages.forEach(message -> {
			Graphics2D g2d = (Graphics2D)canvas.getGraphics();
            g2d.setColor(Color.RED);
            g2d.drawLine(message.getX1(), message.getY1(), message.getX2(), message.getY2());
            frame.repaint();
		});
	}

	private void prepareDrawing() {
		JPanel p = new JPanel() {
			Point lastPoint = null;
	        {
	            addMouseListener(new MouseAdapter() {
	                public void mousePressed(MouseEvent e) {
	                    lastPoint = e.getPoint();
	                }

	                public void mouseReleased(MouseEvent e) {
	                }
	            });
	            
            	addMouseMotionListener(new MouseMotionAdapter() {
	                public void mouseMoved(MouseEvent e) {
	                    
	                }

	                public void mouseDragged(MouseEvent e) {
	                    Point point = e.getPoint();
	                    drawMessages.onNext(new DrawMessage(lastPoint, point));
	                    lastPoint = point;
	                }
	            });
	            
	        }
	        public void paint(Graphics g) {
	            super.paint(g);
	            Graphics2D g2d = (Graphics2D)g;
	            g2d.drawImage(canvas, null, null);
	        }
	    };
	    frame.add(p);
	}

	public PublishSubject<DrawMessage> getDrawMessages() {
		return drawMessages;
	}

	public static void main(String[] args) {
		Whiteboard whiteboard = new Whiteboard();
		whiteboard.getDrawMessages().forEach(System.out::println);
	}
	
	
}
